

#include "wf_client.h"
#include <string.h>

#include "freertos/FreeRTOS.h"
#include "freertos/task.h"
#include "freertos/event_groups.h"
#include "esp_system.h"
#include "esp_wifi.h"
#include "esp_event.h"
#include "esp_log.h"
#include "esp_err.h"
#include "nvs_flash.h"

#include "lwip/err.h"
#include "lwip/sys.h"


static EventGroupHandle_t s_wifi_event_group;

#define WIFI_CONNECTED_BIT BIT0
#define WIFI_FAIL_BIT      BIT1

#define  WIFI_CLIENT_TRY_COUNT   10
#define  USER_LOG   printf

static int s_retry_num = 0;

static void event_handler(void* arg, esp_event_base_t event_base,
                                int32_t event_id, void* event_data){

USER_LOG("AP event base %s,  id %d", event_base, event_id);

    if (event_base == WIFI_EVENT && event_id == WIFI_EVENT_STA_START) {
         esp_wifi_connect();
       } 
    else if (event_base == WIFI_EVENT && event_id == WIFI_EVENT_STA_CONNECTED) {
         esp_wifi_set_ps(WIFI_PS_NONE);
         USER_LOG("sta connected");
     }
    else if (event_base == WIFI_EVENT && event_id == WIFI_EVENT_STA_DISCONNECTED) {
//        show_mode(INITIAL_LED_MODE_CL);
        if (s_retry_num < WIFI_CLIENT_TRY_COUNT) {
            esp_wifi_connect();
            s_retry_num++;
            USER_LOG("retry to connect to the AP, n= %d", s_retry_num);
          } 
        else {
            xEventGroupSetBits(s_wifi_event_group, WIFI_FAIL_BIT);
            USER_LOG("too many fail try connect, restart ");
//          restart ESP32			
//            esp_restart();
        }
        USER_LOG("connect to the AP fail");
    } 
      else if (event_base == IP_EVENT && event_id == IP_EVENT_STA_GOT_IP) {
        ip_event_got_ip_t* event = (ip_event_got_ip_t*) event_data;
        USER_LOG("got ip: " IPSTR "", IP2STR(&event->ip_info.ip));
        s_retry_num = 0;
        xEventGroupSetBits(s_wifi_event_group, WIFI_CONNECTED_BIT);
//        get_my_ip_addr(0);
//        show_mode(CLIENT_LED_MODE_NO);
    };
}


esp_netif_t* esp_netif_create_default_wifi_sta_my(void){
    esp_netif_config_t cfg = ESP_NETIF_DEFAULT_WIFI_STA();
    esp_netif_t *netif = esp_netif_new(&cfg);
    assert(netif);
    esp_netif_attach_wifi_station(netif);
    esp_wifi_set_default_wifi_sta_handlers();
    return netif;
}

esp_netif_t* ni_client;

const char hostname[]="my_host_name";
const char ssid[]="wifi_net_name";
const char pass[]="wifi_password";

#define MAX_TUNE_STR_SIZE  32


void init_in_client_mode(void){
   esp_event_handler_instance_t instance_any_id;
   esp_event_handler_instance_t instance_got_ip;
   wifi_config_t wifi_config;

//   dev_ap_mode=0;
   memset(&wifi_config, 0, sizeof(wifi_config));

    s_wifi_event_group = xEventGroupCreate();

    ESP_ERROR_CHECK(esp_netif_init());

    ESP_ERROR_CHECK(esp_event_loop_create_default());

    ni_client=esp_netif_create_default_wifi_sta();

    wifi_init_config_t cfg = WIFI_INIT_CONFIG_DEFAULT();
    ESP_ERROR_CHECK(esp_wifi_init(&cfg));


    ESP_ERROR_CHECK(esp_netif_set_hostname(ni_client, hostname));

    ESP_ERROR_CHECK(esp_event_handler_instance_register(WIFI_EVENT,
                                                        ESP_EVENT_ANY_ID,
                                                        &event_handler,
                                                        NULL,
                                                        &instance_any_id));
    ESP_ERROR_CHECK(esp_event_handler_instance_register(IP_EVENT,
                                                        IP_EVENT_STA_GOT_IP,
                                                        &event_handler,
                                                        NULL,
                                                        &instance_got_ip));

    wifi_config = (wifi_config_t) {
        .sta = {
            .ssid = "",
            .password ="",
            /* Setting a password implies station will connect to all security modes including WEP/WPA.
             * However these modes are deprecated and not advisable to be used. Incase your Access point
             * doesn't support WPA2, these mode can be enabled by commenting below line */
	         .threshold.authmode = WIFI_AUTH_OPEN,

            .pmf_cfg = {
                .capable = true,
                .required = false
            },
        },
    };
    
    strlcpy((char*)wifi_config.sta.ssid, (char*)ssid, MAX_TUNE_STR_SIZE);
    strlcpy((char*)wifi_config.sta.password, (char*)pass, MAX_TUNE_STR_SIZE);
    wifi_config.sta.threshold.authmode=WIFI_AUTH_WPA2_PSK;

    ESP_ERROR_CHECK(esp_wifi_set_mode(WIFI_MODE_STA) );
    ESP_ERROR_CHECK(esp_wifi_set_config(WIFI_IF_STA, &wifi_config) );

    ESP_ERROR_CHECK(esp_wifi_start() );

    USER_LOG("wf inited in client mode");
//    esp_netif_get_hostname(ni_client, &hn);
//    printf("host name:  %s\n", hn);
//    print_dhcp_status();
}

